/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cxf.io;


import java.io.ByteArrayInputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.cxf.helpers.IOUtils;

public class DelegatingInputStream extends FilterInputStream {
    protected boolean cached;

    public DelegatingInputStream(InputStream is) {
        super(is);
    }


    public void setInputStream(InputStream inputStream) {
        in = inputStream;
    }
    public InputStream getInputStream() {
        return in;
    }

    /**
     * Read the entire original input stream and cache it.  Useful
     * if switching threads or doing something where the original
     * stream may not be valid by the time the next read() occurs
     */
    public void cacheInput() {
        if (!cached) {
            CachedOutputStream cache = new CachedOutputStream();
            try {
                InputStream origIn = in;
                IOUtils.copy(in, cache);
                if (cache.size() > 0) {
                    in = cache.getInputStream();
                } else {
                    in = new ByteArrayInputStream(new byte[0]);
                }
                cache.close();
                origIn.close();
            } catch (IOException e) {
                //ignore
            }
            cached = true;
        }
    }

}