/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cxf.jca.core.resourceadapter;

import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.util.logging.Logger;

import org.apache.cxf.common.logging.LogUtils;

/**
 * allow URI resources from the root of the RAR to be specified as properties
 */
public class Handler extends URLStreamHandler {
    private static final Logger LOG = LogUtils.getL7dLogger(Handler.class);

    public URLConnection openConnection(URL someUrl) {

        return new URLConnection(someUrl) {
            public void connect() {
                LOG.fine("connect");
            }

            public InputStream getInputStream() throws java.io.IOException {
                LOG.fine("getInputStream, path=" + url.getPath());
                InputStream is = Handler.class.getClassLoader().getResourceAsStream(url.getPath());
                if (is == null) {
                    throw new java.io.IOException(url.getPath()
                                                  + " Not Found by getResourceAsStream(), ClassLoader="
                                                  + Handler.class.getClassLoader());
                }
                return is;
            }
        };
    }
}
