/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cxf.jaxrs.client.spring;

import java.lang.annotation.Annotation;
import java.util.Collection;
import java.util.Map;

import javax.ws.rs.ext.Provider;

import org.apache.cxf.common.util.ClasspathScanner;
import org.apache.cxf.common.util.StringUtils;
import org.apache.cxf.jaxrs.client.Client;
import org.apache.cxf.jaxrs.client.JAXRSClientFactoryBean;
import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.service.factory.ServiceConstructionException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;

public class JaxRsWebClientConfiguration extends AbstractJaxRsClientConfiguration {

    @Value("${cxf.jaxrs.client.classes-scan-packages:}")
    private String scanPackages;

    @Bean
    protected Client jaxRsWebClient() {
        return super.createClient();
    }

    @Override
    protected void setJaxrsResources(JAXRSClientFactoryBean factory) {
        factory.setServiceClass(WebClient.class);

        if (!StringUtils.isEmpty(scanPackages)) {
            try {
                final Map< Class< ? extends Annotation >, Collection< Class< ? > > > classes =
                    ClasspathScanner.findClasses(scanPackages, Provider.class);

                factory.setProviders(
                    JAXRSClientFactoryBeanDefinitionParser.getProviders(context, classes.get(Provider.class)));
            } catch (Exception ex) {
                throw new ServiceConstructionException(ex);
            }
        }
    }


}
