/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.cxf.ws.eventing.integration.eventsink;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Logger;

import org.apache.cxf.common.logging.LogUtils;
import org.apache.cxf.ws.eventing.integration.notificationapi.CatastrophicEventSink;
import org.apache.cxf.ws.eventing.integration.notificationapi.EarthquakeEvent;
import org.apache.cxf.ws.eventing.integration.notificationapi.FireEvent;

public class TestingEventSinkImpl implements CatastrophicEventSink {

    public static final AtomicInteger RECEIVED_EARTHQUAKES = new AtomicInteger(0);
    public static final AtomicInteger RECEIVED_FIRES = new AtomicInteger(0);

    protected static final Logger LOG = LogUtils.getLogger(TestingEventSinkImpl.class);


    @Override
    public void earthquake(EarthquakeEvent ev) {
        LOG.info("Event sink received an event: " + ev.toString());
        RECEIVED_EARTHQUAKES.incrementAndGet();
    }

    @Override
    public void fire(FireEvent ev) {
        LOG.info("Event sink received an event: " + ev.toString());
        RECEIVED_FIRES.incrementAndGet();
    }
}
