/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.cxf.systest.jaxrs.discovery;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;

import org.apache.cxf.systest.jaxrs.discovery.sub.BookStoreInterface;
import org.apache.cxf.systest.jaxrs.validation.BookWithValidation;

public class BookStore implements BookStoreInterface {
    @POST
    @Path("/books")
    @Valid
    public BookWithValidation addBook(@NotNull @FormParam("id") String id,
            @FormParam("name") String name) {
        return new BookWithValidation(name, id);
    }

    @GET
    @Path("/book/{id}")
    @Valid
    public BookWithValidation getBook(@NotNull @PathParam("id") String id) {
        return new BookWithValidation("", id);
    }
}
